@@ -10,7 +10,6 @@ import (
 	"strconv"
 	"time"
 
-	"github.com/containers/common/pkg/strongunits"
 	"github.com/containers/podman/v5/pkg/machine/define"
 	"github.com/containers/storage/pkg/fileutils"
 )
@@ -33,14 +32,6 @@ func NewQemuBuilder(binary string, options []string) QemuCmd {
 	return append(q, options...)
 }
 
-// SetMemory adds the specified amount of memory for the machine
-func (q *QemuCmd) SetMemory(m strongunits.MiB) {
-	serializedMem := strconv.FormatUint(uint64(m), 10)
-	// In order to use virtiofsd, we must enable shared memory
-	*q = append(*q, "-object", fmt.Sprintf("memory-backend-memfd,id=mem,size=%sM,share=on", serializedMem))
-	*q = append(*q, "-m", serializedMem)
-}
-
 // SetCPUs adds the number of CPUs the machine will have
 func (q *QemuCmd) SetCPUs(c uint64) {
 	*q = append(*q, "-smp", strconv.FormatUint(c, 10))
